//===============================================================================
// Microsoft Aspiring Software Architects Program
// E-Appointments - Case Study Implementation
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Text;
using System.Collections.Generic;
using System.Transactions;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql;
using EAppointments.BMS.DataAccess;

namespace EAppointments.BMS.UnitTests.BusinessLogic
{
    [TestClass]
    public class GenerateAppointmentSlotsTestFixture : BaseTestFixture
    {
        /// <summary>
        /// Generate Slots by a Provider for a Particular ClinicType.
        /// </summary>
        [TestMethod]
        public void GenerateAppointmentSlots()
        {
            // Search for a Provider
            ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria(null, null);
            providerCriteria.ClinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26"); // Allergy
            providerCriteria.SpecialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");  // Paediatrics
            Provider provider = BaseTestFixture.application.Find(providerCriteria)[0];

            ProviderClinicType pct = provider.GetProviderClinicTypes()[0];

            SlotSearchCriteria slotCriteria = new SlotSearchCriteria(null);
            slotCriteria.Status = SlotStatus.All;
            slotCriteria.ProviderId = provider.Id;
            slotCriteria.SpecialtyId = providerCriteria.SpecialtyId;
            slotCriteria.ClinicTypeId = providerCriteria.ClinicTypeId;

            Slot[] slots = BaseTestFixture.application.Find(slotCriteria);
            int before = slots.Length;
            
            // Generate Slots
            DateTime startDate = DateTime.Now.AddDays(1);
            DateTime endDate = DateTime.Now.AddDays(2);

            pct.GenerateSlots(startDate, endDate);

            // Check if the slots have been generated
            slots = BaseTestFixture.application.Find(slotCriteria);
            int after = slots.Length;

            int count = GetWeekdays(pct.Weekdays, startDate, endDate);

            Assert.AreEqual(after, before + count);

        }

        /// <summary>
        /// Generate Slots by a Provider for a Particular ClinicType for dates that have already passed.
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GenerateAppointmentSlotsForPastDates()
        {
            // Search for a Provider
            ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria(null, null);
            providerCriteria.ClinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26"); // Allergy
            providerCriteria.SpecialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");  // Paediatrics
            Provider provider = BaseTestFixture.application.Find(providerCriteria)[0];

            ProviderClinicType pct = provider.GetProviderClinicTypes()[0];

            // Generate Slots
            pct.GenerateSlots(DateTime.Now.AddDays(-5), DateTime.Now.AddDays(-1));

        }

        /// <summary>
        /// Generate Slots by a Provider for a Particular ClinicType for inconsistent dates.
        /// </summary>
        [TestMethod]
        [ExpectedException(typeof(ArgumentException))]
        public void GenerateAppointmentSlotsForInconsistentDates()
        {
            // Search for a Provider
            ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria(null, null);
            providerCriteria.ClinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26"); // Allergy
            providerCriteria.SpecialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");  // Paediatrics
            Provider provider = BaseTestFixture.application.Find(providerCriteria)[0];

            ProviderClinicType pct = provider.GetProviderClinicTypes()[0];

            // Generate Slots
            pct.GenerateSlots(DateTime.Now.AddDays(5), DateTime.Now.AddDays(1));

        }

        /// <summary>
        /// Generate Slots by a Provider for a Particular ClinicType for  range of dates, some of which have already
        /// passed.
        /// </summary>
        [TestMethod]
        public void GenerateAppointmentSlotsForRangeOfDates()
        {
            // Search for a Provider
            ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria(null, null);
            providerCriteria.ClinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26"); // Allergy
            providerCriteria.SpecialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");  // Paediatrics
            Provider provider = BaseTestFixture.application.Find(providerCriteria)[0];

            ProviderClinicType pct = provider.GetProviderClinicTypes()[0];

            SlotSearchCriteria slotCriteria = new SlotSearchCriteria(null);
            slotCriteria.Status = SlotStatus.All;
            slotCriteria.ProviderId = provider.Id;
            slotCriteria.SpecialtyId = providerCriteria.SpecialtyId;
            slotCriteria.ClinicTypeId = providerCriteria.ClinicTypeId;

            Slot[] slots = BaseTestFixture.application.Find(slotCriteria);
            int before = slots.Length;
            
            // Generate Slots
            DateTime startDate = DateTime.Now.AddDays(-1);
            DateTime endDate = DateTime.Now.AddDays(2);

            pct.GenerateSlots(startDate, endDate);

            slots = BaseTestFixture.application.Find(slotCriteria);
            int after = slots.Length;

            int count = GetWeekdays(pct.Weekdays, startDate, endDate);
            
            Assert.AreEqual(after, before + count);

        }

        private int GetWeekdays(int weekdays, DateTime startDate, DateTime endDate)
        {
            int count = 0;

            if (startDate.Date < DateTime.Now.Date)
                startDate = DateTime.Now;

            for (DateTime currDate = startDate.Date; currDate <= endDate.Date; currDate = currDate.AddDays(1))
            {
                if ((weekdays & 1) > 0 && currDate.DayOfWeek == DayOfWeek.Saturday)
                {
                    count++;
                }

                if ((weekdays & 2) > 0 && currDate.DayOfWeek == DayOfWeek.Friday)
                {
                    count++;
                }

                if ((weekdays & 4) > 0 && currDate.DayOfWeek == DayOfWeek.Thursday)
                {
                    count++;
                }

                if ((weekdays & 8) > 0 && currDate.DayOfWeek == DayOfWeek.Wednesday)
                {
                    count++;
                }

                if ((weekdays & 16) > 0 && currDate.DayOfWeek == DayOfWeek.Tuesday)
                {
                    count++;
                }

                if ((weekdays & 32) > 0 && currDate.DayOfWeek == DayOfWeek.Monday)
                {
                    count++;
                }

                if ((weekdays & 64) > 0 && currDate.DayOfWeek == DayOfWeek.Sunday)
                {
                    count++;
                }
            }
            return count;
        }
    }
}
