//===============================================================================
// Microsoft Aspiring Software Architects Program
// E-Appointments - Case Study Implementation
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================

using System;
using System.Text;
using System.Collections.Generic;
using System.Transactions;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using Microsoft.Practices.EnterpriseLibrary.Data;
using Microsoft.Practices.EnterpriseLibrary.Data.Sql;
using EAppointments.BMS.DataAccess;

namespace EAppointments.BMS.UnitTests.BusinessLogic
{
    [TestClass]
    public class SelectServiceProviderTestFixture : BaseTestFixture
    {
        /// <summary>
        /// Search for Providers on the basis of SpecialtyId
        /// </summary>
        [TestMethod]
        public void SearchServiceProviderOnSpecialty()
        {
            Guid specialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");    //Paediatrics
            ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria(null, specialtyId);

            Provider[] providerList = BaseTestFixture.application.Find(providerCriteria);
            Assert.IsTrue(providerList.Length > 0);

            foreach (Provider provider in providerList)
            {
                ProviderClinicType[] providerClinicTypeList = provider.GetProviderClinicTypes();

                int numCount = 0;
                foreach (ProviderClinicType providerClinicType in providerClinicTypeList)
                {
                    if (providerClinicType.ClinicType.Parent.Id == specialtyId)
                        numCount++;
                }

                Assert.IsTrue(numCount > 0);
            }
        }

        /// <summary>
        /// Search for Providers on the basis of ClinicTypeId
        /// </summary>
        [TestMethod]
        public void SearchServiceProviderOnClinicType()
        {
            Guid clinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");    //Allergy
            ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria(null, null);
            providerCriteria.ClinicTypeId = clinicTypeId;

            Provider[] providerList = BaseTestFixture.application.Find(providerCriteria);
            Assert.IsTrue(providerList.Length > 0);

            foreach (Provider provider in providerList)
            {
                ProviderClinicType[] providerClinicTypeList = provider.GetProviderClinicTypes();

                int numCount = 0;
                foreach (ProviderClinicType providerClinicType in providerClinicTypeList)
                {
                    if (providerClinicType.ClinicType.Id == clinicTypeId)
                        numCount++;
                }
                Assert.IsTrue(numCount > 0);
            }
        }

        /// <summary>
        /// Search for Providers on the basis of Keywords
        /// </summary>
        [TestMethod]
        [Ignore]
        public void SearchServiceProviderOnKeywords()
        {
            ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria(null, null);
            
            // Search on Alternative Services
            providerCriteria.Keywords = "Transport Available";

            Provider[] providerList = BaseTestFixture.application.Find(providerCriteria);
            
            Assert.IsTrue(providerList.Length > 0);

            foreach (Provider provider in providerList)
            {
                Assert.IsTrue(provider.AlternativeServices.ToLower().Contains(providerCriteria.Keywords.ToLower()));
            }

            // Search on Conditions Treated
            providerCriteria.Keywords = "Spinal Disorders";

            providerList = BaseTestFixture.application.Find(providerCriteria);

            Assert.IsTrue(providerList.Length > 0);

            foreach (Provider provider in providerList)
            {
                Assert.IsTrue(provider.ConditionsTreated.ToLower().Contains(providerCriteria.Keywords.ToLower()));
            }

            // Search on Procedures Performed
            providerCriteria.Keywords = "Physiotherapy";

            providerList = BaseTestFixture.application.Find(providerCriteria);

            Assert.IsTrue(providerList.Length > 0);

            foreach (Provider provider in providerList)
            {
                Assert.IsTrue(provider.ProceduresPerformed.ToLower().Contains(providerCriteria.Keywords.ToLower()));
            }

            // Search on Exclusions
            providerCriteria.Keywords = "Do not perform MRIs";

            providerList = BaseTestFixture.application.Find(providerCriteria);

            Assert.IsTrue(providerList.Length > 0);

            foreach (Provider provider in providerList)
            {
                Assert.IsTrue(provider.Exclusions.ToLower().Contains(providerCriteria.Keywords.ToLower()));
            }

        }

        /// <summary>
        /// Search for Providers on Proximity
        /// </summary>
        [TestMethod]
        public void SearchServiceProviderOnProximity()
        {
            string zipCode = "411001";
            double withinMiles = (double)1;

            ProviderSearchCriteria providerCriteria = new ProviderSearchCriteria(null, null);
            providerCriteria.ZipCode = zipCode;
            providerCriteria.WithinMiles = withinMiles;

            Provider[] providerList = BaseTestFixture.application.Find(providerCriteria);
            Assert.IsTrue(providerList.Length > 0);

            foreach (Provider provider in providerList)
            {
                Assert.IsTrue(provider.Proximity <= 1);
            }
        }


    }
}
