//===============================================================================
// Microsoft Aspiring Software Architects Program
// E-Appointments - Case Study Implementation
//===============================================================================
// Copyright  Microsoft Corporation.  All rights reserved.
// THIS CODE AND INFORMATION IS PROVIDED "AS IS" WITHOUT WARRANTY
// OF ANY KIND, EITHER EXPRESSED OR IMPLIED, INCLUDING BUT NOT
// LIMITED TO THE IMPLIED WARRANTIES OF MERCHANTABILITY AND
// FITNESS FOR A PARTICULAR PURPOSE.
//===============================================================================
// The example companies, organizations, products, domain names,
// e-mail addresses, logos, people, places, and events depicted
// herein are fictitious.  No association with any real company,
// organization, product, domain name, email address, logo, person,
// places, or events is intended or should be inferred.
//===============================================================================


#region Namespaces Referred 
using System;
using System.Text;
using System.Data;
using System.Collections.Generic;
using System.Transactions;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EAppointments.BMS.DataAccess;
#endregion

namespace EAppointments.BMS.UnitTests
{
    /// <summary>
    /// Unit Testing for EAppointments.BMS
    /// </summary>
    [TestClass]
    public class DataAccessTestFixture
    {
        DbLib db;
        private TransactionScope scope;

        public DataAccessTestFixture()
        {
            this.db = new DbLib();
        }

        [TestInitialize]
        public void TestInit()
        {
            scope = new TransactionScope(TransactionScopeOption.Required);
        }

        [TestCleanup]
        public void TestCleanup()
        {
            scope.Dispose();
        }


        /// <summary>
        /// Test Method for User Create, Retrieve, Update, Delete.
        /// </summary>
        [TestMethod]
        public void UserCRUD()
        {
            Guid userId = Guid.NewGuid();
            String userName = "testUser1";
            String password = "testPassword";
            String firstName = "Test";
            String lastName = "User";
            String email = "testUser@somecompany.com";
            Guid refUserId = Guid.NewGuid();
            
            // Create User
            db.UserSave(userId, userName, password, firstName, lastName, email, 1, refUserId);
            
            DataTable dt = db.UserGet(userId, null);
            Assert.IsTrue(dt.Rows.Count == 1);
            Assert.AreEqual(dt.Rows[0][0].ToString(), userId.ToString());
            Assert.AreEqual(dt.Rows[0][1].ToString(), userName);
            Assert.AreEqual(dt.Rows[0][2].ToString(), password);
            Assert.AreEqual(dt.Rows[0][3].ToString(), firstName);
            Assert.AreEqual(dt.Rows[0][4].ToString(), lastName);
            Assert.AreEqual(dt.Rows[0][5].ToString(), email);
            Assert.AreEqual(dt.Rows[0][6].ToString(), "1");
            Assert.AreEqual(dt.Rows[0][7].ToString(), refUserId.ToString());

            // Retrieve User
            dt = db.UserGet(null, userName);
            Assert.IsTrue(dt.Rows.Count == 1);
            Assert.IsTrue(dt.Rows[0][1].ToString() == userName);

            // Delete User
            int before = db.UserGet(null, null).Rows.Count;
            int n = db.UserDelete(userId);
            Assert.IsTrue(n == 1);

            int after = db.UserGet(null, null).Rows.Count;
            Assert.IsTrue(before == after + 1);
        }

        /// <summary>
        /// Test Method for Session Create, Retrieve, Update, Delete.
        /// </summary>
        [TestMethod]
        public void SessionCRUD()
        {
            Guid userId = Guid.NewGuid();
            String userName = "jdoe";
            String password = "jdoe";
            String firstName = "John";
            String lastName = "Doe";
            String email = "jdoe@somecompany.com";
            Guid refUserId = Guid.NewGuid();

            // Create User
            db.UserSave(userId, userName, password, firstName, lastName, email, 1, refUserId);

            // Create Session
            Guid sessionId = Guid.NewGuid();
            db.SessionSave(sessionId, userId, 20, DateTime.Now, DateTime.Now);

            // Retrieve Session
            DataTable dt = db.SessionGet(sessionId);
            Assert.AreEqual(dt.Rows.Count, 1);
            Assert.AreEqual(dt.Rows[0][0].ToString(), sessionId.ToString());
            Assert.AreEqual(dt.Rows[0][1].ToString(), userId.ToString());
            Assert.AreEqual(dt.Rows[0][2].ToString(), "20");

            // Delete Session
            int before = db.SessionGet(null).Rows.Count;
            int n = db.SessionDelete(sessionId, DateTime.Now);
            Assert.IsTrue(n == 1);

            int after = db.SessionGet(null).Rows.Count;
            Assert.IsTrue(before == after + 1);

            // Delete User
            db.UserDelete(userId);
        }

        /// <summary>
        /// Test Method for Referrer Create, Retrieve, Update, Delete
        /// </summary>
        [TestMethod]
        public void ReferrerCRUD()
        {
            Guid referrerId = Guid.NewGuid();
            String firstName = "James";
            String clinicName = "Ruby Hall";
            String lastName = "Brown";
            String email = "jbrown@bmshospital.com";

            // Create Referrer
            db.ReferrerSave(referrerId, firstName, clinicName, lastName, email);

            // Retrieve Referrer
            DataTable dt = db.ReferrerGet(referrerId, null, null);
            Assert.IsTrue(dt.Rows.Count == 1);
            Assert.IsTrue(dt.Rows[0][0].ToString() == referrerId.ToString());

            // Delete Referrer
            int before = db.ReferrerGet(referrerId, null, null).Rows.Count;
            int n = db.ReferrerDelete(referrerId);
            Assert.IsTrue(n == 1);
            int after = db.ReferrerGet(referrerId, null, null).Rows.Count;
            Assert.IsTrue(before == after + 1);
        }

        /// <summary>
        /// Test Method for Patient Create, Retrieve, Update, Delete
        /// </summary>
        [TestMethod]
        public void PatientCRUD()
        {
            Guid referrerId = Guid.NewGuid();
            String firstNameReferrer = "James";
            String clinicName = "Ruby Hall";
            String lastNameReferrer = "Brown";
            String email = "jbrown@bmshospital.com";

            // Create Referrer
            db.ReferrerSave(referrerId, firstNameReferrer, clinicName, lastNameReferrer, email);

            Guid patientId = Guid.NewGuid();
            String patientNo = "1222322331";
            String title = "Mr.";
            String firstName = "Jon";
            String lastName = "Doe";
            Char gender = 'M';
            DateTime dob = DateTime.Now.AddYears(-34);
            String addressLine1 = "232, Waterman Avenue";
            String addressLine2 = "9th Main Road";
            String city = "Boston";
            String state = "Masachussets";
            String zipCode = "3233343";
            String country = "USA";
            bool consent = true;
            String contactNo = "132322323223";
            String patientEmail = "jdoe@somecompany.com";

            // Create Patient
            db.PatientSave(patientId, patientNo, title, firstName, lastName, gender, dob, addressLine1, addressLine2, city, state, country, zipCode, consent, contactNo, patientEmail, referrerId);

            // Retrieve Patient
            DataTable dt = db.PatientGet(patientId, null, null, null);
            Assert.IsTrue(dt.Rows.Count == 1);
            Assert.IsTrue(dt.Rows[0][0].ToString() == patientId.ToString());
            int before = db.PatientGet(null, null, null, null).Rows.Count;
            
            // Delete Patient
            int n = db.PatientDelete(patientId);
            Assert.IsTrue(n == 1);
            int after = db.PatientGet(null, null, null, null).Rows.Count;
            Assert.IsTrue(before == after + 1);

            // Delete Referrer
            db.ReferrerDelete(referrerId);


        }

        /// <summary>
        /// Test Method for Clinic Type Lookup
        /// </summary>
        [TestMethod]
        public void ClinicTypeCRUD()
        {
            Guid clinicTypeId = new Guid("E11EC421-1262-4CF6-82E7-003ACD01AA26");
            Guid specialtyId = new Guid("2E85B151-99B7-43A7-AF74-BBF88A035A45");
            DataTable dt = db.ClinicTypeGet(null, specialtyId);
            Assert.IsTrue(dt.Rows[0][0].ToString().Trim() == clinicTypeId.ToString());

        }

        /// <summary>
        /// Test Method for Provider Create, Retrieve, Update, Delete
        /// </summary>
        [TestMethod]
        public void ProviderCRUD()
        {
            
            Guid providerId = Guid.NewGuid();
            String name = "BMS Hospital";
            String location = "Mumbai";
            String organization = "Apollo Hospitals";
            String email = "apollo@hospital.com";
            Double latitude  = 98.093433D;
            Double longitude = 34.34333D;
            String conditionsTreated = "HyperTension, Diabetes, Asthama, Dental Problems";
            String proceduresPerformed = "Surgically Assisted Orthodontics";
            String exclusions = "";
            String alternativeServices = "Transport Available";
            Guid clinicTypeID = new Guid("77BAF929-DD4D-4480-9A80-1CBC54CA450A");
            Guid providerClinicTypeId = Guid.NewGuid();

            // Create Provider
            db.ProviderSave(providerId, name, location, organization, email, latitude, longitude, conditionsTreated, proceduresPerformed, exclusions, alternativeServices);
            db.ProviderClinicTypeSave(providerClinicTypeId, providerId, clinicTypeID, 0, 0, DateTime.Now, DateTime.Now, 1);
            
            // Retrieve Provider
            DataTable dt = db.ProviderGet(providerId, null, clinicTypeID, null, null, null);
            Assert.IsTrue(dt.Rows.Count == 1);
            Assert.IsTrue(dt.Rows[0]["Id"].ToString() == providerId.ToString());
            int before = db.ProviderGet(null, null, null, null, null, null).Rows.Count;
            

            // TODO: Keyword Search


            // Delete Provider
            int m = db.ProviderClinicTypeDelete(providerId, clinicTypeID);
            Assert.IsTrue(m == 1);
            int n = db.ProviderDelete(providerId);
            Assert.IsTrue(n == 1);
            int after = db.ProviderGet(null, null, null, null, null, null).Rows.Count;
            Assert.IsTrue(before == after + 1);

        }

        /// <summary>
        /// Test Method for Slot Create, Retrieve, Update, Delete
        /// </summary>
        [TestMethod]
        public void SlotCRUD()
        {
            Guid slotId = Guid.NewGuid();
            int ubrnNo = 122333221;
            Guid providerId = Guid.NewGuid();
            String name = "BMS Hospital";
            String location = "Mumbai";
            String organization = "Apollo Hospitals";
            String email = "apollo@hospital.com";
            Double latitude = 98.093433D;
            Double longitude = 34.34333D;
            String conditionsTreated = "HyperTension, Diabetes, Asthama, Dental Problems";
            String proceduresPerformed = "Surgically Assisted Orthodontics";
            String exclusions = "";
            String alternativeServices = "Transport Available";
            Guid clinicTypeID = new Guid("77BAF929-DD4D-4480-9A80-1CBC54CA450A");
            Guid specialtyId = new Guid("E58FC643-F6AB-411C-943C-EE400241CB5D");
            Guid providerClinicTypeId = Guid.NewGuid();

            // Create Provider and ProviderClinicType
            db.ProviderSave(providerId, name, location, organization, email, latitude, longitude, conditionsTreated, proceduresPerformed, exclusions, alternativeServices);
            db.ProviderClinicTypeSave(providerClinicTypeId, providerId, clinicTypeID, 0, 0, DateTime.Now, DateTime.Now, 1);
            
            
            // Create Slot
            db.SlotSave(slotId, providerId, specialtyId, clinicTypeID, DateTime.Now, DateTime.Now, ubrnNo, (int)SlotStatus.Booked);

            // Retrieve Slot
            DataTable dt = db.SlotGet(slotId, null, null, null, null, null, null, ubrnNo, (int)SlotStatus.Booked);
            Assert.IsTrue(dt.Rows.Count == 1);
            Assert.IsTrue(dt.Rows[0][0].ToString() == slotId.ToString());
            int before = db.SlotGet(null, null, null, null, null, null, null, null, null).Rows.Count;

            // Delete Slot
            int n = db.SlotDelete(slotId);
            Assert.IsTrue(n == 1);
            int after = db.SlotGet(null, null, null, null, null, null, null, null, null).Rows.Count;
            Assert.IsTrue(before == after + 1);
            
            // Delete Provider and ProviderClinicType
            int m = db.ProviderClinicTypeDelete(providerId, clinicTypeID);
            Assert.IsTrue(m == 1);
            n = db.ProviderDelete(providerId);
            Assert.IsTrue(n == 1);
        }

        /// <summary>
        /// Test Method for Appointment Create, Retrieve, Update, Delete
        /// </summary>
        [TestMethod]
        public void AppointmentCRUD()
        {

            // Create Referrer
            Guid referrerId = Guid.NewGuid();
            String name = "James Brown";
            String clinicName = "Ruby Hall";
            String role = "GP";
            String email = "jbrown@bmshospital.com";
            db.ReferrerSave(referrerId, name, clinicName, role, email);

            // Create Patient
            Guid patientId = Guid.NewGuid();
            String patientNo = "1222322331";
            String title = "Mr.";
            String firstName = "Jon";
            String lastName = "Doe";
            Char gender = 'M';
            DateTime dob = DateTime.Now.AddYears(-34);
            String addressLine1 = "232, Waterman Avenue";
            String addressLine2 = "9th Main Road";
            String city = "Boston";
            String state = "Masachussets";
            String zipCode = "3233343";
            String country = "USA";
            bool consent = true;
            String contactNo = "132322323223";
            String patientEmail = "jdoe@somecompany.com";
            db.PatientSave(patientId, patientNo, title, firstName, lastName, gender, dob, addressLine1, addressLine2, city, state, country, zipCode, consent, contactNo, patientEmail, referrerId);

            // Create Provider and ProviderClinicType
            Guid providerId = Guid.NewGuid();
            String providerName = "BMS Hospital";
            String location = "Mumbai";
            String organization = "Apollo Hospitals";
            String providerEmail = "apollo@hospital.com";
            Double latitude = 98.093433D;
            Double longitude = 34.34333D;
            String conditionsTreated = "HyperTension, Diabetes, Asthama, Dental Problems";
            String proceduresPerformed = "Surgically Assisted Orthodontics";
            String exclusions = "";
            String alternativeServices = "Transport Available";
            Guid clinicTypeID = new Guid("77BAF929-DD4D-4480-9A80-1CBC54CA450A");
            Guid specialtyId = new Guid("E58FC643-F6AB-411C-943C-EE400241CB5D");
            
            Guid providerClinicTypeId = Guid.NewGuid();
            db.ProviderSave(providerId, providerName, location, organization, providerEmail, latitude, longitude, conditionsTreated, proceduresPerformed, exclusions, alternativeServices);
            db.ProviderClinicTypeSave(providerClinicTypeId, providerId, clinicTypeID, 0, 0, DateTime.Now, DateTime.Now, 1);

            // Create Slot
            Guid slotId = Guid.NewGuid();
            int ubrnNo = 122333221;
            db.SlotSave(slotId, providerId, specialtyId, clinicTypeID, DateTime.Now, DateTime.Now, ubrnNo, (int)SlotStatus.Booked);

            // Create Appointment
            db.AppointmentSave(ubrnNo, patientId, referrerId, providerId, clinicTypeID, slotId, DateTime.Now, DateTime.Now.AddDays(1), DateTime.Now.AddHours(1).AddDays(1), DateTime.Now, null, null, null, 1, null, null, null);

            // Retrieve Appointment
            DataTable dt = db.AppointmentGet(ubrnNo, patientId, referrerId, providerId, 1, null, null, null);
            Assert.IsTrue(dt.Rows.Count == 1);
            Assert.IsTrue(dt.Rows[0][0].ToString().Trim() == ubrnNo.ToString());
            int before = db.AppointmentGet(null, null, null, null, null, null, null, null).Rows.Count;

            // Delete Appointment
            int m = db.AppointmentDelete(ubrnNo);
            Assert.IsTrue(m == 1);
            int after = db.AppointmentGet(null, null, null, null, null, null, null, null).Rows.Count;
            Assert.IsTrue(before == after + 1);

            // Delete Slot
            db.SlotDelete(slotId);

            // Delete ProviderClinicType
            db.ProviderClinicTypeDelete(providerId, clinicTypeID);

            // Delete Provider
            db.ProviderDelete(providerId);

            // Delete Patient
            db.PatientDelete(patientId);

            // Delete Referrer
            db.ReferrerDelete(referrerId);

        }
    
    }
}
